        PAGE    60,132
name    TWEAK
;       [ - condensed  Toshiba P351
        TITLE   TWEAK Version 1.6
comment |

=================

TWEAK.COM

Copyright (c) Roedy Green Canadian Mind Products (1990)
Shareware that may be freely distributed for non-military use only.
Adjusts diskette drive parameters to speed up disk access.

Tweak.Com  /M500 /H25 /K2000 /R6 /Q

  /M500   = 500 ms = 0.5 seconds to wait for _M_otor to come up to speed
            (will be rounded to nearest 1/8 second)

  /H25    = 25 ms for _H_ead to settle after moving to a new track

  /K2000  = 2032 ms = 2.032 seconds to _K_eep the disk spinning
            (will be rounded to nearest tick (1/18.2 second).

  /R6     = 6 ms step _Rate
            (will be rounded to nearest 2 ms.)

  /Q      = _Q_uiet.  Suppress all displays.  Use this for your
            autoexec.bat file to avoid disturbing the earthlings.

Note BIOS uses a common table for both A: and B:.  You cannot configure
the drives separately.

Please report bugs and problems to:

Roedy Green
Canadian Mind Products
#208 - 525 Ninth Street
New Westminster BC Canada
V5H 2N6
tel:(604) 777-1804
mailto:roedy@mindprod.com
http://mindprod.com

Version 1.0 1990 May 2
        - beta test released to ten BIXen.

Version 1.1 1990 May 5
        - new lower limit on /M
        - use of word "conservative" instead of "standard"
        - /Q option to suppress display.
        - no params at all just lists current settings.
        - tsr form now uses only about 144 bytes.
        - normally Tweak is not a TSR at all and thus uses no bytes.
        - frees SET environment and tail end of PSP.

Version 1.2 1990 May 15
        - new lower limit on /H

Version 1.3 1990 June 27
        - properly frees the SET environment, missing INT 21h
        - now closes StdIn and StdOut to conserve handles.
        - less amibiguous messages about original, old, new, current,
          and conservative settings of the parameters.

Version 1.4 1993 June 1
        - change of address
        - added address to banner

Version 1.5 1996 October 25
        - embed POB 707 Quathiaski Cove address
        - get rid of centred messages, all now left justified.


Version 1.6 1998 November 8
        - embed Barker address

;==============================================================

;       C O N V E N T I O N S

Subroutines may trash any registers they please except CS: DS:
ES: SS: and of course the outputs.

Some of the routines save more registers than this convention
requires.  This is because they were canibalized from other
environments where there were stricter rules.  Speed is not an
issue here.

| ; end of comment

;==============================================================
;       E Q U A T E S

; for info on standard values see AT tech ref page 5-192
;                                 XT tech ref page A-46

;----

MinM    equ     100d    ; minimum  M in ms /M motor start time
SafeM   equ     1000d   ; Safe M in ms
                        ; 500 ms XT, 1000 ms AT
                        ; Here given in ms.
                        ; In table coded in 1/8 second increments
                        ; e.g. code 4, code 8, measured in 1/8 secs
MaxM    equ     1500d   ; maximum  M in ms

;----

MinH    equ     1d      ; minimum  H in ms /H head settle time
SafeH   equ     25d     ; Safe H in ms
                        ; 25=XT std, 20=AT std for 360K drive, 15=AT std 1.2MB
MaxH    equ     30d     ; maximum  H in ms

;----

MinK    equ     250d    ; minimum  K in ms /K keep motor on
SafeK   equ     2032d   ; Safe K in ms
                        ; Here given in ms.
                        ; In table coded in ticks.  2 secs = 37 ticks
MaxK    equ     5000d   ; maximum  K in ms

;----

MinR    equ     2d      ; minimum  R in ms /R step rate
SafeR   equ     6d      ; Safe R in ms
                        ; Here given in ms.
                        ; Encoded in a bizarre way in the table
                        ; in multiples of 2.
MaxR    equ     8d      ; maximum  R in ms

;---

RamDParmAdr equ 5Ch     ; build RAM-based disk parm block in PSP on top
                        ; of first unused FCB at offset 5Ch
                        ; This is only used if disk parm block is not
                        ; in RAM already.


;==============================================================

; M A C R O S

CR      MACRO   ; Carriage return line feed
        DB 0dh,0ah
        ENDM

;======

EOS     MACRO   ; marks end of display string
        DB 0dh,0ah,'$'
        ENDM

;======
;       S T R U C T U R E S

DParm   Struc                   ; structure of DOS disk parm block
                                ; int 1E vector is  pointer to this block
                                ; 11d bytes long.  Sometimes called DISK_BASE
Specify1        db      ?       ; /R = CFh
                                ; ((16-(step/2)) shl 4) OR 0F
                                ; e.g. CF C=8 ms  F=unload time
Specify2        db      ?
KeepOn          db      ?       ; /K = 25h time to keep on in timer ticks
                                ; 37d = 2 secs
SectSizeCode    db      ?       ; will be 2 to mean 512 bytes
Eot             db      ?       ; last sector on track e.g. 8, 9, 15
Gap             db      ?       ; 2Ah inter sector gap
DTL             db      ?       ; I haven't the foggiest idea
GapFormat       db      ?       ; 50h gap length for format
FillFormat      db      ?       ; F6h fill byte for format
HeadSettle      db      ?       ; /H = 19h head settle time in ms
                                ; 15d 1.2 MB AT
                                ; 20d 360KB
MotorOn         db      ?       ; /M = 4 motor on start in 1/8 seconds

Dparm   EndS
;=============================================================

;       P R O C E D U R E S

CSEG    segment
        assume CS:CSEG, DS:CSEG, ES:CSEG, SS:CSEG
                                        ; Since this is a .COM program
        org 100h                        ; Likewise

start:

Main    Proc    near
;       Initialize, then, possibly, Terminate and Stay Resident
        call    Parse           ; parse 4 command switches on
                                ; command line
                                ; This is a dry run just be sure
                                ; command line is ok.
                                ; If we fail, screen space is limited.
                                ; Parse might exit via Abort
        lea     dx,CopyRightMsg ; CopyRight banner
        call    Say             ; display centered
        call    ShowSafe        ; Show Safe parms

        test    Changes,1
        jnz     DoChanges
ShowOnly:
        call    ShowCur         ; Show existing parms
        mov     ax, 4c00h       ; ERRORLEVEL = 0
        int     21h             ; Die

DoChanges:
        call    ShowOld         ; Show existing parms
        call    Parse           ; Parse again, this time override OLD values
        call    IsItInRAM       ; Decide if RAM or ROM based parm block
        test    RAMFlag,1
        jnz     WasRAM          ; jmp if RAM

WasROM:                         ; Was ROM, we need to go TSR
        call    ReDirect        ; copy and set up vector to point to our RAM
        call    PokeValues      ; Poke new values into param block
        call    ReCal           ; reset floppy controller
        call    ShowNew         ; Show existing parms
        call    TSR             ; Terminate and Stay Resident
                                ; Die

WasRAM:                         ; was RAM, just use existing block
        call    PokeValues      ; Poke new values into parm block
        call    ReCal           ; reset floppy controller
        call    ShowNew         ; Show new parms
        mov     ax, 4c00h       ; ERRORLEVEL = 0
        int     21h             ; normal exit to dos
Main    EndP

;==============================================================

abort   Proc    Near
;       If user blows it, we still treat him well!
        mov     Q,0             ; turn displays back on
        lea     dx,CopyRightMsg ; CopyRight banner
        call    Say             ; display centered
        lea     dx,TroubleMsg
        call    Say
        call    ShowSafe        ; Show Safe parms
        call    ShowCur         ; Show existing parms
        mov     ax, 4c04h       ; ERRORLEVEL = 4
        int     21h             ; Die
abort   EndP

;============================================

ShowOld Proc    near
        ; show the old parms before we change them
        lea     dx,OldMsg
        call    Say
        call    PeekValues
        call    ShowState
        ret

ShowOld EndP

;==============================================================
ShowCur Proc    near
        ; show the current parms
        lea     dx,CurMsg
        call    Say
        call    PeekValues
        call    ShowState
        ret

ShowCur EndP

;==============================================================

PeekValues      Proc near
;       Get current disk parameters and convert them to ms
;       then store them in variables M H K R

        push    ES              ; save old ES

        mov     ax,351Eh        ; get vector for int 1E
        int     21H             ; vector in ES:bx

;---
                                ; /M PeekValues
        mov     AL,ES:[bx.MotorOn]
        xor     ah,ah
        mov     cl,125d
        mul     cl              ; convert from 1/8 sec to ms
        mov     M,ax

;---
                                ; /H PeekValues
        mov     al,ES:[bx.HeadSettle]
        xor     ah,ah
        mov     H,ax            ; already in ms

;---
                                ; /K PeekValues
        mov     al,ES:[bx.KeepOn]
        xor     ah,ah
        mov     cx,54925d       ; Convert from ticks to ms
        mul     cx              ; there are 18.20648 ticks/sec
        add     ax,500d
        adc     dx,0d
        mov     cx,1000d        ; 1193180/65536 to be precise.
        div     cx              ; Each tick is thus 54.92549 ms
                                ; for greater accuracy we
                                ; mult by 54925 and divide by 1000
                                ; to round division we add 500
        mov     K,ax

;---
                                ; /R PeekValues
        mov     al,ES:[bx.Specify1]
                                ; 'CF'h -> 8 ms
        shr     al,1            ; extract the C
        shr     al,1            ; can't use shr al,4 on XT
        shr     al,1
        shr     al,1
        sub     al,16d          ; subtract it from 16
        neg     al
        shl     al,1            ; multiply by 2
        xor     ah,ah
        mov     R,ax

;---

        pop     ES              ; restore ES
        ret
PeekValues      EndP

;==============================================================

ShowSafe Proc   near
        ; show the Safe parameter settings.
        ; This routine will be cleverer in future to examine hardware
        ; before making blanket suggestions.
        lea     dx,SafeMsg
        call    Say
        mov     M,SafeM
        mov     H,SafeH
        mov     K,SafeK
        mov     R,SafeR
        call    ShowState
        ret

ShowSafe EndP

;==============================================================

ShowNew Proc    near

; show the vector as it will be after we change it

        lea     dx,NewMsg
        call    Say
;       parameters are already set the way they will be
        call    PeekValues              ; we do this to see effects
                                        ; of granularity of units
        call    ShowState
        ret

ShowNew EndP

;==============================================================

ShowState Proc  near
;       shows parameters stored in variables M H K R -- not the DOS params

        lea     dx,MMsg
        call    Say
        mov     ax,M
        call    SayDec
        lea     dx,MsMsg
        call    Say

        lea     dx,HMsg
        call    Say
        mov     ax,H
        call    SayDec
        lea     dx,MsMsg
        call    Say

        lea     dx,KMsg
        call    Say
        mov     ax,K
        call    SayDec
        lea     dx,MsMsg
        call    Say

        lea     dx,RMsg
        call    Say
        mov     ax,R
        call    SayDec
        lea     dx,MsMsg
        call    Say

        ret

ShowState EndP

;==============================================================

Parse   Proc    near
;       Parse the command line for the optional  /M500 /H25 /K2000 /R6 /Q

        call    CommandLine     ; string addr ES:di, length cx.
        jcxz    NullParms

        mov     ParmIndex,1     ; start parsing with the 1st parm
                                ; note start with 1 not 0!
                                ; We don't want the the program name.
        jmp     Parseloop

NullParms:
NoMoreParms:
        ret                     ; we are done

Parseloop:
        call    CommandLine     ; get first parm
                                ; string=ES:di, length=cx
        mov     bx,ParmIndex
        call    NthParm         ; work left to right
        jcxz    NoMoreParms     ; null param means no more
                                ; e.g. DS:si points to /M500
                                ; cx is length of that piece
        lodsb
        dec     cx              ; cx contains length of rest of parm
                                ; e.g. M500 without the /
        cmp     al,'/'          ; ensure / present
        jne     BadParms
        lodsb                   ; split on M H K R
        dec     cx              ; cx contains length of rest of parm
                                ; e.g. 500 without /M
        call    ToUC            ; convert m -> M, h -> H etc
        cmp     al,'M'
        JE      ProcessM
        cmp     al,'H'
        je      ProcessH
        cmp     al,'K'
        je      ProcessK
        cmp     al,'R'
        je      ProcessR
        cmp     al,'Q'
        je      ProcessQ
                                ; something else, give up.
BadParms:
        jmp     Abort

ProcessM:                       ; handle /M500
        call    ASCIIToBin      ; dx:ax is result
        mov     bx,MinM         ; low bound
        mov     cx,MaxM         ; high bound
        call    Corral
        mov     M,ax
        mov     Changes,1
        jmp     Next

ProcessH:                       ; parse /H25
        call    ASCIIToBin      ; dx:ax is result
        mov     bx,MinH         ; low bound
        mov     cx,MaxH         ; high bound
        call    Corral
        mov     H,ax
        mov     Changes,1
        jmp     Next

ProcessK:                       ; parse /K2000
        call    ASCIIToBin      ; dx:ax is result
        mov     bx,MinK         ; low bound
        mov     cx,MaxK         ; high bound
        call    Corral
        mov     K,ax
        mov     Changes,1
        jmp     Next

ProcessR:                       ; parse /R6
        call    ASCIIToBin      ; dx:ax is result
        mov     bx,MinR         ; low bound
        mov     cx,MaxR         ; high bound
        call    Corral
        add     ax,1            ; round UP to next even number
        and     ax,0FEh         ; mask off low bit
        mov     R,ax            ; If we did not do this, it
        mov     Changes,1
        jmp     Next            ; it would round down later.

ProcessQ:
        mov     Q,1             ; Make a note to suppress display
        jmp     next            ; /Q does not count as a change

Next:
        inc     ParmIndex       ; bump loop counter
        jmp     Parseloop       ; loop till hit null param

ParmIndex       dw      0
Parse   EndP

;==============================================================

CommandLine Proc        near
;       Gets command line string into ES:di
;       Command line does not include the program name.
;       ES already set since we are a COM file
;       Sets cx to length of command line.
;       Trashes no other registers
                                ; counted string at HEX 80
                                ; contains command line.
                                ; It has no trailing null.
        mov     di,81h
        xor     CH,CH
        mov     CL,DS:80h       ; cx contains length of command
        ret
CommandLine EndP

;==============================================================

NthParm Proc    near
;       Parses string for Nth Parameter delimited by blanks
;       e.g.  "  /M250 /H25" with bx=1 would give string "/M250"
;       on entry:
;       ES:di - string
;       cx - length of string
;       bx - which parm wanted 1=parm1 2=parm2 etc.
;       NthParm only finds one parm per call.
;       On exit ES:si points to string and cx is its length.
;       If there is no parm, the length will be 0.
;       It also handles multiple leading/trailing blanks on parms.
        mov     al,20h          ; al = blank  -- the search char
Parmloop:
;       Remove leading blanks on parm
        jcxz    NullParm        ; jump if null string
        repe    scasb           ; scan ES:di forwards till hit non blank
                                ; di points just after it
                                ; cx is one too small, or 0 if none found
        je      NullParm        ; jump if entire string was blank
        inc     cx              ; cx is length of remainder of string
        dec     di              ; di points to non-blank
        mov     si,di           ; remember start of string
;       Search for terminating blank on parm
        jcxz    NullParm        ; jump if null string
        repne   scasb           ; scan ES:di forwards till hit blank
                                ; di points just after it
                                ; cx is one too small, or 0 if none found
        jne     NoBlank         ; jump if entire string was non blank
        inc     cx              ; cx is length of remainder of string
        dec     di              ; backup di to point to blank at string end
NoBlank:
                                ; di=addr tail end of command string,
                                ; cx=len tail end of command string
                                ; si=addr parm just parsed
;       Major loop for each parm
        dec     bx
        jnz     Parmloop        ; loop once for each parm

        mov     cx,di
        sub     cx,si           ; cx is length of parameter.
        ret
NullParm:                       ; was no nth parameter
        mov     cx,0
        ret
NthParm EndP

;==============================================================

ASCIIToBin      Proc    near
;       on entry DS:si points to start of decimal ASCII string of digits.
;       String should contain only the decimal digits 0..9.
;       If there are bad chars we abort.
;       cx contains length of string > 0
;       On exit dx:ax contains binary equivalent of string.
        push    bx              ; save regs we will trash
        push    cx
        push    si
        push    di
        xor     ax,ax
        mov     dx,ax           ; accumulate in dx:ax

OneDigitloop:

; multiply accumulated number by 10
                                ; Works because 10*X = 2*X + 8*X
                                ; use di:bx as working register
        shl     ax,1
        rcl     dx,1            ; dx:ax = x*2
        mov     bx,ax
        mov     di,dx
        shl     bx,1
        rcl     di,1
        shl     bx,1
        rcl     di,1            ; di:bx = x*8
        add     bx,ax
        adc     di,dx           ; di:bx = x*2 + x*8

; add on the next digit
        xor     dx,dx
        mov     AH,DH
        lodsb                   ; get next digit in al
        sub     al,"0"          ; convert 1 digit ASCII to bin
        JB      NumTrouble      ; ensure 0..9
        cmp     al,9
        JA      NumTrouble
        add     ax,bx           ; add it onto accumulated product
        adc     dx,di

        loop    OneDigitloop    ; loop once for each digit

        pop     di              ; restore regs
        pop     si
        pop     cx
        pop     bx

        ret
NumTrouble:
;       Some problem, give standard Error message.
        jmp     Abort
;
ASCIIToBin      EndP

;==============================================================

SayDec PROC     NEAR
;       call with number in range 0..65535 in ax
;       converts it to ASCII and displays it on the screen
;       If it is 0 shows as 0.
;       leading 0's are suppressed.
;       field is exactly wide enough to hold the number.
;       No leading or trailing spaces.  DISPLAYED IN DECIMAL
        Test    Q,1             ; See if supposed to suppress output
        jz      SayDecLoud
        ret
SayDecLoud:
        push    di              ; preserve di
        mov     bx,10D
        mov     di,5            ; work right to left building digits at PAD
SayDecLoop:
        dec     di
        xor     dx,dx           ; dx:ax / 10
        div     bx              ; ax=quot dx=remdr
        add     dl,'0'          ; convert digit to ASCII
        mov     pad[di],dl
        or      ax,ax
        jnz     SayDecLoop
SayDecDone:
;       Number is ready
        lea     dx,pad[di]
        call    Say
        pop     di
        ret
SayDec  endp

;========================

Corral  Proc    near
;       on entry ax=number bx=low bound cx=high bound
;       on exit ax is safely corralled into the range bx:cx
        cmp     ax,bx
        jae     BigEnough       ; use unsigned compares
        mov     ax,bx
BigEnough:
        cmp     cx,ax
        jae     SmallEnough
        mov     ax,cx
SmallEnough:
        ret
Corral  EndP

;==============================================================

IsItInRAM       Proc    near
        ; determines if disk parameters in RAM or ROM
        ; sets RAMflag true if in ram
        push    ES
        mov     ax,351Eh        ; get vector for int 1E
        int     21H             ; vector in ES:bx
        mov     al,ES:[bx]
        not     al
        mov     ES:[bx],al      ; try to store onto rom
        cmp     al,ES:[bx]      ; see if it changed
        jne     InROM
InRAM:
        mov     RAMflag,1       ; was RAM
        not     al              ; put it back
        mov     ES:[bx],al
        pop     ES
        ret
InROM:
        mov     RAMflag,0       ; was ROM

        pop     ES
        ret

IsItInRAM       EndP

;==============================================================

PokeValues      Proc    near
        ; poke new values into the disk parameter block
        push    ES              ; save old ES

        mov     ax,351Eh        ; get vector for int 1E
        int     21H             ; vector in ES:bx

        mov     ax,M            ; poke /M convert from ms to 1/8 second units
        mov     cl,125d
        add     ax,62d          ; round by adding 62
        div     cl
        mov     ES:[bx.MotorOn],al

        mov     ax,H            ; poke /H already in ms
        mov     ES:[bx.HeadSettle],al

        mov     ax,K            ; poke /K convert from ms to ticks
        mov     cx,182d         ; there are 18.20648 ticks/sec
        mul     cx              ; 1193180/65536 to be precise.
        add     ax,5000d
        adc     dx,0
        mov     cx,10000d       ; Each ms is 0.018206 ticks.
        div     cx              ; for greater accuracy we
                                ; mult by 182 and divide by 10000
                                ; to round the division we add 5000
        mov     ES:[bx.KeepOn],al

        mov     ax,R            ; poke /R
                                ; convert 8 ms into CF
        shr     al,1            ; divide by 2, to get 4
        sub     al,16d          ; subtract from 16 to get 12
        neg     al              ; now have C
        shl     al,1            ; can't shl al,4 on XT
        shl     al,1
        shl     al,1
        shl     al,1            ; now have C0
        or      al,0fh          ; now have CF
        mov     ES:[bx.Specify1],al

        pop     ES              ; restore ES

        ret
PokeValues EndP

;==============================================================

ReDirect        Proc    near
; make copy of old parm block to one at RamDParmAdr in PSP
; then redirect the int 1E to RamDParmAdr in PSP
        push    DS
        push    ES
        mov     ax,351Eh        ; get old int 1E vector ES:BX
        int     21H             ; set vector from DS:dx
        mov     si,bx
        mov     di,RamDParmAdr
        mov     ax,ES           ; swap ES: and DS:
        mov     bx,DS
        mov     ES,bx
        mov     DS,ax
                                ; DS:SI points to old  ES:DI points to new
        mov     cx,(size Dparm) ; CX is length = 11d
        rep     movsb           ; copy the string
;       Set up new int 1E vector
        mov     ax,CS
        mov     DS,ax
        mov     dx,RamDParmAdr
        mov     ax,251Eh        ; set vector for 1E
        int     21H             ; set vector from DS:dx
        pop     ES
        pop     DS
        ret
ReDirect        EndP

;==============================================================

Recal   Proc    near
;       reset so new parms take effect
;       forces recalibrate on next i/o.
;       does not actually recalibrate now.
        mov     dx,0
        mov     ax,0            ; int 13 function 0
        int     13h
        ret
Recal   EndP

;==============================================================

TSR     Proc    near
;       Terminate and Stay Resident
        mov     ES,DS:[2Ch]     ; pointer to environment
        mov     ah,49h          ; free RAM allocated to SET environment
        int     21h

        MOV     AH,3Eh          ; Close StdIn
        MOV     BX,0            ; Avoid holding onto a handle if
        INT     21H             ; foolish user typed TWEAK <Myfile

        MOV     AH,3Eh          ; Close StdOut
        MOV     BX,1            ; Avoid holding onto a handle if
        INT     21H             ; user typed TWEAK >NUL:

        call    CommandLine     ; string addr ES:di, length cx.
        mov     dx,di           ; count how long in bytes piece
                                ; we need is
        add     dx,cx
        shr     dx,1            ; divide by 16 to get paras
        shr     dx,1
        shr     dx,1
        shr     dx,1            ; dx has paras to keep resident
        inc     dx              ; to ensure trail nul included,
                                ; also to round up.
                                ; could have added 1+15d prior
                                ; We keep PSP including FCB-1 and
                                ; the command line.
        mov     ax,3100h        ; Terminate and Stay Resident
                                ; Errorlevel =0
                                ; Die
        int     21h
TSR     EndP

;============================================


ToUC    Proc    near
;       converts char in al to upper case
        cmp     al,'a'
        jb      FineAsIs
        cmp     al,'z'
        ja      FineAsIs
        sub     al,20H          ; convert a to A
FineAsIs:
        ret
ToUc    EndP

;============================================

Say     Proc    near
;       display message on screen
;       on entry DX contains address of $ delimited message.
        Test    Q,1             ; See if supposed to suppress output
        jz      SayLoud
        ret
SayLoud:
        mov     AH,09h
        int     21h
        ret
Say     EndP

;============================================
comment   no longer used
SayCent Proc
;       on entry dx points to a string to display
;       it may contain CrLfs in it.  It is terminated by $.
;       We display each line centred on a 80 column screen.
        Test    Q,1             ; See if supposed to suppress output
        jz      SayCentLoud
        ret
SayCentLoud:
LineLoop:
        mov     si,dx   ; si points to first char
        mov     di,dx   ; di save secord copy
        sub     bx,bx   ; count chars in line, 0 to start
                        ; si points to first char of line
CharLoop:
        lodsb           ; get first char
        cmp     al,'$'
        je      FoundEnd
        cmp     al,13d
        je      FoundEol
        inc     bx      ; found an ordinary char, count it.
        JMP     CharLoop
FoundEol:
                        ; found Cr, print centred line.
        mov     cx,80d  ; calc # spaces needed for centering
        sub     cx,bx   ; bx is count of real chars in line
        shr     cx,1    ; divide by 2
        and     cx,3Fh  ; just in case line too long.
        jcxz    SpacesDone
                        ; cx now contains number of spaces we want to emit.
SpaceLoop:
        mov     DL,' '
        mov     ah,02
        int     21H     ; emit enough spaces to centre the line.
        LOOP    SpaceLoop
SpacesDone:
        mov     cx,bx   ; bx contains length of string
        add     cx,2    ; account for CrLf at the end
        mov     dx,di   ; cx=count dx=addr
        mov     bx,1    ; handle 1 - standard output
        mov     ax,4000H ; ah=40 signifies write by handle
        int     21H     ; Write the string itself
        inc     si      ; point just past the LF
        mov     dx,si   ; point to start of next string.
        JMP     LineLoop; process the next line of text
FoundEnd:
                        ; We are done.  All lines have CrLf before $
        ret             ; so we need not process any tail end.
SayCent EndP
 ; end comment
;============================================

;       V A R I A B L E S

PAD     DB      "00000$"
                        ; where numeric output built by SayDec

; To avoid MASM phase errors or silly BYTE PTR, we make flags 2 bytes

RamFlag dw      0       ; true if disk parameters currently in ram.
                        ; false if in ROM and we need to move them
                        ; and go TSR.
Changes dw      0       ; 0=no changes 1=changes desired to settings


M       dw      0       ; time is ms to turn _M_otor on
H       dw      0       ; _H_ead settle time in ms
K       dw      0       ; time to _K_eep disk running in ms
R       dw      0       ; time to step _R_ate for one track in ms
Q       dw      0       ; 0=display 1=quietly suppress

;======================================
;
;       D I S P L A Y   S T R I N G S

CopyrightMsg    label byte
        DB      ' Tweak 1.6 ۲'
        CR
        DB      'Shareware to adjust your floppy drives for peak performance.'
        CR
        DB      'Copyright (c) 1990-1999 Roedy Green Canadian Mind Products'
        CR
        DB      '#208 - 525 Ninth Street, New Westminster, BC Canada V3M 5T9',13,10
        DB      'tel:(604) 777-1804   mailto:roedy@mindprod.com   http://mindprod.com',13,10
        DB      'May be freely distributed and used for any purpose except military.'
        EOS


TroubleMsg      label byte
        CR
        DB ' Error in Command line ۲'
        CR
        db 'Try something like this:'
        CR
        CR
        db 'Tweak /M1000 /H25 /K2032 /R6 /Q'
        db 07d
        EOS

OldMsg  label   byte
        CR
        db      "Old settings:"
        EOS

SafeMsg label   byte
        CR
        db      "Suggested Conservative settings:"
        EOS

CurMsg label byte
        CR
        db      "Current settings:"
        EOS

NewMsg  label   byte
        CR
        DB      "New settings:"
        EOS

MMsg    db      '  /M time to wait for motor on is $'
HMsg    db      '  /H time to wait for heads to settle is $'
KMsg    db      '  /K time to keep motor running is $'
RMsg    db      '  /R time for the step rate is $'

MsMsg   db      ' ms',10,13,'$' ; append millisecond units

CSEG ends
end start
